%function DSP_PS2
% The goals of this problemset are to demonstate
% Convolution Theorem and the Wiener-Khinchin theorem, and to introduce 
% Matlab filter design and time-frequency analysis

%% Convolution Theorem 
clc;
% Generate simulated data
x = rand(1,5); % signal
h = rand(1,3); % kernel
Fs = 1; % sampling rate


Nx = numel(x);  % number of samples in signal
Nh = numel(h);  % number of samples in kernel
Nc = Nx+Nh-1; % number of samples in convolution


% time domain convolution
x_pad = [zeros(1,Nh-1) x zeros(1,Nh-1)];

c1 = zeros(1,numel(Nc)); % prealocate output
for n = 1:Nc
  c1(n) = dot(x_pad(n:n+numel(h)-1)', flip(h,2));
end

% matlab conv
c2 = conv(x,h,'full');

% frequency domain convolution
%zero-pad 
N = numel(x)+numel(h)-1;
x_pad = [x zeros(1,N-numel(x))];
h_pad = [h zeros(1,N-numel(h))];

c3 = ifft(fft(x_pad).*fft(h_pad));%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



fprintf('time domain convolution: \n');disp(c1)
fprintf('matlab conv: \n');disp(c2)
fprintf('frequency domain convolution: \n');disp(c3)

%% Wiener-Khinchin theorem
clc;
% Compute the PSD with fft  (just like last week)
% Big hint 2: use an N-point fft where N = Nx*2;

psd1 = abs(fft(x,Nx*2)).^2; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psd1 = psd1./sqrt(Nx); % Enforce Parseval's Rule
frq = Fs.*(0:Nx-1)./Nx;

% Compute the PSD with the autocorrelation
% hint: doc xcorr, and keep using 

r1 = xcorr(x);
psd2 = abs((fft(r1, Nx*2)));%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psd2(1:2:end);
psd2 = psd2./sqrt(Nx); % Enforce Parseval's Rule


% Given that taking the complex conjugate of a frequency domain signal is 
% equivelent to the time reversing a time-domain signal, compute the autocorrelation in the frequecy domain

% hint 1: doc conj N-point fft where N = Nx*2


r2 = ifft( fft(x,Nx*2) .* conj(fft(x,Nx*2)) );%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r2 = [r2(end-Nx+2:end)  r2(1:Nx)]; % unwrap


% Compute the PSD using the frequency domain autocorrelation

psd3 = fft(x,Nx*2) .* conj(fft(x,Nx*2));%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

psd3 = psd3./sqrt(Nx); % Enforce Parseval's Rule


fprintf('matlab xcorr: \n');disp(r1)
fprintf('frequency domain autocorrelation: \n');disp(r2)
fprintf('\n\n')
fprintf('fft^2: \n');disp(psd1)
fprintf('fft(xcorr): \n');disp(psd2)
fprintf('fft(frequency domain autocorrelation): \n');disp(psd3)


%% Build a short-time fft
Fs = 64; % sampling rate (Hz)
T = 64; % ~one minutes
t = 1/Fs:1/Fs:T;
x = chirp(t,1,T*.5,Fs/2,'log'); % signal
N = numel(x); % duration (samples)
n = N/128; % window length (samples)

figure(1)
subplot(3,1,1)
plot(t,x)
ylabel('amplitude')
xlabel('time (s)')
title('signal')
xlim([t(1) t(end)])

%%
stfft = cell(1,N-n);
for winstrt = 1:N-n
  fprintf('processing %i of %i ...\n',winstrt,N-n)
  win = winstrt:round(winstrt+n-1);
  stfft{winstrt} = abs(fft(hann(n).*x(win)',n)).^2./sqrt(n);
  stfft{winstrt} = stfft{winstrt}(1:n/2);
end
frq = Fs.*(0:(n-1))./n;
frq = frq(1:n/2);

% Build a spectrogram 
% horizontally concatentate stfft's
subplot(3,1,2)
imagesc(t(1:N-n),frq,horzcat(stfft{:}))
set(gca,'ydir','normal')
ylabel('Frequency (Hz)')
xlabel('time (s)')
title('short-time-fft')

% What happens when you halve or double the window size?
% The spectrogram precision is proportional to the window size (within these bounds)
% Precision costs time.

% What happens when you remove the Hann window function?
% What should be a continuous amplitude chirp becomes artifactually punctuated. 


% That happens that the time-edges of the spectrogram? frequency-edges?
% Artifacts exist there.

%% Use the Hilbert transform to extract narrow-band power

Nf = numel(frq);
filtHilb = cell(1,Nf-1);
for ifrq = 2:Nf
  fprintf('processing %i of %i ...\n',ifrq,Nf)
   passband = [frq(ifrq) frq(ifrq)+1];
  bpFilt = designfilt('bandpassfir', ...
             'FilterOrder', 20, ...
             'CutoffFrequency1', passband(1), ...
             'CutoffFrequency2', passband(2),...
             'SampleRate', Fs);
   xFilt = filtfilt(bpFilt,x);
   filtHilb{ifrq} = hilbert(xFilt);
end
filtHilb{1} = zeros(size(filtHilb{2}));

% amplitude  = abs(Ax);
% theta = angle(Ax);


% Build a spectrogram 
subplot(3,1,3);cla
imagesc(t,frq,abs(vertcat(filtHilb{:})))
set(gca,'ydir','normal')
ylabel('Frequency (Hz)')
xlabel('time (s)')
title('Filter-Hilbert')

clear bpFilt

% What happens when you halve or double the filter order?

% The spectrogram precision is proportional to the filter order (within these bounds)




%% Filter design

% Load the simulated signal from today's lecture

foo = load('SimSig.mat')

% Use filterDesigner or fdatool to recreate the low-pass, high-pass and 
% bandpass filters from today's lecture.



















